@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -92,6 +92,7 @@ Recordset::Recordset(GRTManager *grtm)
   _id = g_atomic_int_get(&next_id);
   g_atomic_int_inc(&next_id);
 
+  task->desc("Recordset task");
   task->send_task_res_msg(false);
   apply_changes_cb= boost::bind(&Recordset::apply_changes_, this);
   register_default_actions();
@@ -184,7 +185,7 @@ bool Recordset::reset(Recordset_data_storage::Ptr data_storage_ptr, bool rethrow
   }
 
   data_edited();
-  refresh_ui();
+  //refresh_ui(); This is the wrong place for a GUI refresh. Reset is called from many places, including the c-tor.
 
   return res;
 }
@@ -268,6 +269,8 @@ void Recordset::rollback()
 {
   if (!reset(false))
     task->send_msg(grt::ErrorMsg, _("Rollback failed"), _("Rollback recordset changes"));
+  else
+    refresh_ui();
 }
 
 
@@ -1085,7 +1088,7 @@ void Recordset::paste_rows_from_clipboard(ssize_t dest_row)
         mforms::Utilities::show_error("Cannot Paste Row",
                                       strfmt("Number of fields in pasted data doesn't match the columns in the table (%zi vs %zi).\n"
                                              "Data must be in the same format used by the Copy Row Content command.",
-                                             (long)parts.size(), (long)get_column_count()),
+                                             parts.size(), get_column_count()),
                                       "OK");
 
         if (rows_changed && row != rows.begin())
