@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,10 +18,12 @@
  */
 
 #import "TabViewDockingDelegate.h"
-#include "NSString_extras.h"
+#import "NSString_extras.h"
+
 #include "mforms/appview.h"
 
-TabViewDockingPointDelegate::TabViewDockingPointDelegate(NSTabView *tabView, const std::string &type) : _tabView(tabView), _type(type)
+TabViewDockingPointDelegate::TabViewDockingPointDelegate(NSTabView *tabView, const std::string &type)
+: _tabView(tabView), _type(type)
 {
 }
 
@@ -34,25 +36,21 @@ mforms::AppView *TabViewDockingPointDelegate::appview_for_view(NSView *view)
 }
 
 
-bool TabViewDockingPointDelegate::handle_close_clicked(mforms::AppView *view)
-{
-  if (view->on_close())
-  {
-    undock_view(view);
-    return true;
-  }
-  return false;
-}
-
-
 bool TabViewDockingPointDelegate::close_all()
 {
   for (NSTabViewItem *item in [_tabView tabViewItems])
   {
-    mforms::AppView *aview = appview_for_view([item view]);
-    if (aview && !handle_close_clicked(aview))
+    mforms::AppView *view = appview_for_view(item.view);
+    if (view != NULL && !view->on_close())
       return false;
   }
+
+  for (NSTabViewItem *item in [_tabView tabViewItems])
+  {
+    mforms::AppView *view = appview_for_view(item.view);
+    if (view != NULL)
+      view->close();
+  }
   return true;
 }
 
@@ -60,7 +58,7 @@ bool TabViewDockingPointDelegate::close_all()
 void TabViewDockingPointDelegate::dock_view(mforms::AppView *view, const std::string &arg1, int arg2)
 {
   id v = view->get_data();
-  NSTabViewItem *tabItem = [[NSTabViewItem alloc] initWithIdentifier: [NSString stringWithFormat: @"appview:%p", view]];
+  NSTabViewItem *tabItem = [[[NSTabViewItem alloc] initWithIdentifier: [NSString stringWithFormat: @"appview:%p", view]] autorelease];
   [tabItem setView: v];
 
   if (arg1 == "" || arg1 == "append")
@@ -68,8 +66,10 @@ void TabViewDockingPointDelegate::dock_view(mforms::AppView *view, const std::st
   else if (arg1 == "prepend")
     [_tabView insertTabViewItem: tabItem atIndex: 0];
 
-  [tabItem release];
-
+  if (view->release_on_add())
+    view->set_release_on_add(false);
+  else
+    view->retain();
   _views[v] = view;
 
   if ([[_tabView delegate] respondsToSelector: @selector(tabView:didSelectTabViewItem:)])
@@ -95,13 +95,14 @@ bool TabViewDockingPointDelegate::select_view(mforms::AppView *view)
 void TabViewDockingPointDelegate::undock_view(mforms::AppView *view)
 {
   NSInteger i = [_tabView indexOfTabViewItemWithIdentifier: [NSString stringWithFormat: @"appview:%p", view]];
-  if (i >= 0)
+  if (i != NSNotFound)
   {
     NSTabViewItem *item = [_tabView tabViewItemAtIndex: i];
     if (item)
     {
       [_tabView removeTabViewItem: item];
       _views.erase(view->get_data());
+      view->release();
     }
   }
 }
@@ -122,10 +123,36 @@ void TabViewDockingPointDelegate::set_view_title(mforms::AppView *view, const st
 }
 
 
-void TabViewDockingPointDelegate::get_size(int *w, int *h)
+std::pair<int, int> TabViewDockingPointDelegate::get_size()
 {
   NSRect frame = [_tabView contentRect];
-  *w = (int)NSWidth(frame);
-  *h = (int)NSHeight(frame);
+  return std::make_pair(NSWidth(frame), NSHeight(frame));
 }
 
+
+mforms::AppView *TabViewDockingPointDelegate::selected_view()
+{
+  id view = [[_tabView selectedTabViewItem] view];
+  if (view)
+  {
+    if (_views.find(view) != _views.end())
+      return _views[view];
+  }
+  return NULL;
+}
+
+
+int TabViewDockingPointDelegate::view_count()
+{
+  return [_tabView numberOfTabViewItems];
+}
+
+
+mforms::AppView *TabViewDockingPointDelegate::view_at_index(int index)
+{
+  id view = [[_tabView tabViewItemAtIndex: index] view];
+
+  if (_views.find(view) != _views.end())
+    return _views[view];
+  return NULL;
+}
