@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -43,6 +43,6 @@
 @property (nonatomic, assign) BOOL drawsBackground;
 @property (nonatomic, retain) NSColor *backgroundColor;
 
-- (NSSize)preferredSize;
+@property (readonly) NSSize preferredSize;
 
 @end
