@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,7 +22,7 @@
 
 @implementation MFFormImpl
 
-- (id)initWithObject:(::mforms::Form*)form
+- (instancetype)initWithObject:(::mforms::Form*)form
                owner:(::mforms::Form*)ownerWindow
 {
   self= [super initWithContentRect:NSMakeRect(100, 100, 1,1)
@@ -115,6 +115,9 @@
   [self setContentSize:size];
 }
 
+- (BOOL)windowShouldClose: (id)sender {
+  return mOwner->can_close();
+}
 
 - (void)windowWillClose:(NSNotification *)notification
 {
