@@ -33,7 +33,7 @@ static struct
 } layout_info[] = 
 {
   {32, 12, 5, 36 - 2 * 7, 9}, // Main
-  {26, 12, 2, 25 - 2 * 7, 8}, // Secondary
+  {26, 12, 2, 25 - 2 * 7, 7}, // Secondary
   {36, 12, 7, 36 - 2 * 7, 7}, // ToolPicker
   {20, 8, 1, 20-2*1, 4}, // Options
   {26, 4, 3, 26-2*3, 2} // Palette
@@ -61,7 +61,7 @@ static NSColor* colorFromHexString(const char* hexcolor)
   return [NSColor colorWithDeviceRed:r / 255.0 green:g / 255.0 blue:b / 255.0 alpha: 1.0];
 }
 
-- (id)initWithColorName: (NSString*)color
+- (instancetype)initWithColorName: (NSString*)color
 {
   self = [super init];
   if (self)
@@ -90,17 +90,24 @@ static NSColor* colorFromHexString(const char* hexcolor)
   ToolBarItem *mOwner;
   BOOL mHorizontal;
 }
-- (id)initWithItemObject:(ToolBarItem*)item;
-- (ToolBarItemType)itemType;
+
+- (instancetype)initWithItemObject:(ToolBarItem*)item NS_DESIGNATED_INITIALIZER;
+
+@property (readonly) mforms::ToolBarItemType itemType;
+@property (readonly) mforms::ToolBarItem *toolBarItem;
+
 - (void)setHorizontal:(BOOL)flag;
-- (ToolBarItem*)toolBarItem;
+
 @end
 
 
 @implementation MFToolBarSeparatorImpl
 
-- (id)initWithItemObject:(ToolBarItem*)item
+- (instancetype)initWithItemObject: (ToolBarItem*)item
 {
+  if (item == nil)
+    return nil;
+
   self = [super initWithFrame: NSMakeRect(0, 0, 1, layout_info[0].height)];
   if (self)
   {
@@ -110,6 +117,16 @@ static NSColor* colorFromHexString(const char* hexcolor)
   return self;
 }
 
+- (instancetype)initWithFrame: (NSRect)frame
+{
+  return [self initWithItemObject: nil];
+}
+
+- (instancetype)initWithCoder: (NSCoder *)coder
+{
+  return [self initWithItemObject: nil];
+}
+
 - (ToolBarItem*)toolBarItem
 {
   return mOwner;
@@ -179,15 +196,18 @@ static NSColor* colorFromHexString(const char* hexcolor)
   ToolBarItem *mOwner;
   BOOL mToolPicker;
 }
-- (id)initWithItemObject:(ToolBarItem*)item;
-- (ToolBarItem*)toolBarItem;
+- (instancetype)initWithItemObject:(ToolBarItem*)item NS_DESIGNATED_INITIALIZER;
+@property (readonly) mforms::ToolBarItem *toolBarItem;
 @end
 
 
 @implementation MFToolBarActionItemImpl
 
-- (id)initWithItemObject:(ToolBarItem*)item
+- (instancetype)initWithItemObject: (ToolBarItem*)item
 {
+  if (item == nil)
+    return nil;
+
   self = [super initWithFrame: NSMakeRect(0, 0, layout_info[0].item_width, layout_info[0].item_width)];
   if (self)
   {
@@ -220,6 +240,16 @@ static NSColor* colorFromHexString(const char* hexcolor)
   return self;
 }
 
+-(instancetype)initWithFrame: (NSRect)frame
+{
+  return [self initWithItemObject: nil];
+}
+
+-(instancetype)initWithCoder: (NSCoder *)coder
+{
+  return [self initWithItemObject: nil];
+}
+
 - (ToolBarItem*)toolBarItem
 {
   return mOwner;
@@ -249,13 +279,19 @@ static NSColor* colorFromHexString(const char* hexcolor)
 - (void)viewDidMoveToSuperview
 {
   MFToolBarImpl *toolbar = (MFToolBarImpl*)[self superview];
-  if (![self alternateImage] && mOwner->get_type() == ToggleItem && [toolbar type] == ToolPickerToolBar)
+  if (![self alternateImage] && mOwner->get_type() == ToggleItem)
   {
     mToolPicker = YES;
     [self setButtonType: NSOnOffButton];
     [self setBordered: NO];
     [self setBezelStyle: NSTexturedSquareBezelStyle];
     [[self cell]  setBackgroundColor: [toolbar backgroundColor]];
+
+    if ([toolbar type] == mforms::SecondaryToolBar)
+    {
+      NSSize tsize = [toolbar frame].size;
+      [self setFrameSize: NSMakeSize(tsize.height, tsize.height)];
+    }
   }
   else
   {
@@ -267,6 +303,7 @@ static NSColor* colorFromHexString(const char* hexcolor)
 - (void)setStringValue: (NSString*)value
 {
   [self setTitle: value];
+  [self sizeToFit];
 }
 
 - (NSString*)stringValue
@@ -278,20 +315,23 @@ static NSColor* colorFromHexString(const char* hexcolor)
 
 //--------------------------------------------------------------------------------------------------
 
-
 @interface MFToolBarLabelItemImpl : NSTextField
 {
   MFToolBarImpl *mToolbar;
   ToolBarItem *mOwner;
 }
-- (id)initWithItemObject:(ToolBarItem*)item;
+- (instancetype)initWithItemObject:(ToolBarItem*)item NS_DESIGNATED_INITIALIZER;
 - (void)setToolbar:(MFToolBarImpl*)toolbar;
-- (ToolBarItem*)toolBarItem;
+@property (readonly) mforms::ToolBarItem *toolBarItem;
 @end
 
 @implementation MFToolBarLabelItemImpl
-- (id)initWithItemObject:(ToolBarItem*)item
+
+- (instancetype)initWithItemObject: (ToolBarItem*)item
 {
+  if (item == nil)
+    return nil;
+
   self = [super initWithFrame: NSMakeRect(0, 0, 200, layout_info[0].item_width)];
   if (self)
   {
@@ -311,6 +351,15 @@ static NSColor* colorFromHexString(const char* hexcolor)
   return self;
 }
 
+-(instancetype)initWithFrame: (NSRect)frame
+{
+  return [self initWithItemObject: nil];
+}
+
+-(instancetype)initWithCoder: (NSCoder *)coder
+{
+  return [self initWithItemObject: nil];
+}
 
 - (void)setToolbar:(MFToolBarImpl *)toolbar
 {
@@ -352,13 +401,17 @@ static NSColor* colorFromHexString(const char* hexcolor)
 {
   ToolBarItem *mOwner;
 }
-- (id)initWithItemObject:(ToolBarItem*)item;
-- (ToolBarItem*)toolBarItem;
+- (instancetype)initWithItemObject:(ToolBarItem*)item NS_DESIGNATED_INITIALIZER;
+@property (readonly) mforms::ToolBarItem *toolBarItem;
 @end
 
 @implementation MFToolBarImageItemImpl
-- (id)initWithItemObject:(ToolBarItem*)item
+
+- (instancetype)initWithItemObject: (ToolBarItem*)item
 {
+  if (item == nil)
+    return nil;
+
   self = [super initWithFrame: NSMakeRect(0, 0, layout_info[0].item_width, layout_info[0].item_width)];
   if (self)
   {
@@ -370,6 +423,16 @@ static NSColor* colorFromHexString(const char* hexcolor)
   return self;
 }
 
+-(instancetype)initWithFrame: (NSRect)frame
+{
+  return [self initWithItemObject: nil];
+}
+
+-(instancetype)initWithCoder: (NSCoder *)coder
+{
+  return [self initWithItemObject: nil];
+}
+
 - (ToolBarItem*)toolBarItem
 {
   return mOwner;
@@ -389,13 +452,17 @@ static NSColor* colorFromHexString(const char* hexcolor)
 {
   ToolBarItem *mOwner;
 }
-- (id)initWithItemObject:(ToolBarItem*)item;
-- (ToolBarItem*)toolBarItem;
+- (instancetype)initWithItemObject:(ToolBarItem*)item NS_DESIGNATED_INITIALIZER;
+@property (readonly) mforms::ToolBarItem *toolBarItem;
 @end
 
 @implementation MFToolBarSearchItemImpl
-- (id)initWithItemObject:(ToolBarItem*)item
+
+- (instancetype)initWithItemObject: (ToolBarItem*)item
 {
+  if (item == nil)
+    return nil;
+
   self = [super initWithFrame: NSMakeRect(0, 0, 200, layout_info[0].item_width)];
   if (self)
   {
@@ -409,6 +476,16 @@ static NSColor* colorFromHexString(const char* hexcolor)
   return self;
 }
 
+-(instancetype)initWithFrame: (NSRect)frame
+{
+  return [self initWithItemObject: nil];
+}
+
+-(instancetype)initWithCoder: (NSCoder *)coder
+{
+  return [self initWithItemObject: nil];
+}
+
 - (ToolBarItem*)toolBarItem
 {
   return mOwner;
@@ -426,14 +503,14 @@ static NSColor* colorFromHexString(const char* hexcolor)
 {
   ToolBarItem *mOwner;
 }
-- (id)initWithItemObject:(ToolBarItem*)item;
-- (ToolBarItem*)toolBarItem;
+- (instancetype)initWithItemObject:(ToolBarItem*)item NS_DESIGNATED_INITIALIZER;
+@property (readonly) mforms::ToolBarItem *toolBarItem;
 @end
 
 @implementation MFToolBarSelectorItemImpl
-- (id)initWithItemObject:(ToolBarItem*)item
+- (instancetype)initWithItemObject:(ToolBarItem*)item
 {
-  self = [super initWithFrame: NSMakeRect(0, 0, 200, layout_info[0].item_width)];
+  self = [super initWithFrame: NSMakeRect(0, 0, 150, layout_info[0].item_width)];
   if (self)
   {
     mOwner = item;
@@ -492,7 +569,7 @@ static NSColor* colorFromHexString(const char* hexcolor)
 
 @implementation MFToolBarImpl
 
-- (id)initWithObject:(ToolBar*)owner type:(ToolBarType)type
+- (instancetype)initWithObject:(ToolBar*)owner type:(ToolBarType)type
 {
   switch (type) 
   {
@@ -595,7 +672,7 @@ static NSColor* colorFromHexString(const char* hexcolor)
   if (index >= (int)[[self subviews] count])
     [self addSubview: view];
   else
-    [self addSubview: view positioned: NSWindowBelow relativeTo: [[self subviews] objectAtIndex: index]];
+    [self addSubview: view positioned: NSWindowBelow relativeTo: [self subviews][index]];
   if (wasEmpty)
   {
     // main toolbar starts with a reduced height if it's empty. so when something is added, it has to
@@ -973,6 +1050,7 @@ static void set_selector_items(ToolBarItem *item, const std::vector<std::string>
       for (std::vector<std::string>::const_iterator iter = items.begin();
            iter != items.end(); ++iter)
         [array addObject: wrap_nsstring(*iter)];
+      [tbitem removeAllItems];
       [tbitem addItemsWithTitles: array];
       [tbitem sizeToFit];
     }
