@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -58,7 +58,9 @@ static BOOL modalHUDRunning = NO;
   modalLoopRunningCond = [[NSCondition alloc] init];
 }
 
-+ (BOOL)runModalHudWithTitle: (NSString*) title andDescription: (NSString*) description cancelAction: (boost::function<bool ()>)cancelAction
++ (BOOL)runModalHudWithTitle: (NSString*) title andDescription: (NSString*) description
+                 notifyReady: (boost::function<void ()>)signalReady
+                cancelAction: (boost::function<bool ()>)cancelAction
 {
   if (instance == nil)
     instance = [[MHudController alloc] init];
@@ -86,7 +88,10 @@ static BOOL modalHUDRunning = NO;
   modalHUDRunning = YES;
   [modalLoopRunningCond signal];
   [modalLoopRunningCond unlock];
-  
+
+  if (signalReady)
+    signalReady();
+    
   NSInteger ret = -1;
   // can't use runModalForWindow because it will just block until some event happens
   // (like mouse move), even after stopModal is called
@@ -101,6 +106,9 @@ static BOOL modalHUDRunning = NO;
   
   [NSApp endModalSession: instance->modalSession];
   instance->modalSession = nil;
+
+  // make sure shared_refs bound to it are not kept dangling
+  instance->cancelAction.clear();
   
   if (ret == NSRunAbortedResponse)
   {
@@ -140,7 +148,7 @@ static BOOL modalHUDRunning = NO;
 
 //--------------------------------------------------------------------------------------------------
 
-- (id) init
+- (instancetype) init
 {
   if ((self= [super init]) != nil)
   {
