@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "base/string_utilities.h"
 #include "base/threading.h"
 
@@ -70,11 +68,11 @@ void ClassRegistry::register_all(GRT *grt)
  for (std::map<std::string,ClassRegistrationFunction>::const_iterator iter= classes.begin();
       iter != classes.end(); ++iter)
  {
-   // register classes only for loaded metaclasses
+   // Register classes only for loaded meta classes.
    if (!grt->get_metaclass(iter->first))
    {
      if (grt->verbose())
-       grt->send_warning("MetaClass "+iter->first+" is registered but was not loaded from a XML");
+       grt->send_warning("MetaClass " + iter->first + " is registered but was not loaded from a XML");
      continue;
    }
    (*iter->second)(grt);
@@ -94,7 +92,7 @@ ClassRegistry *ClassRegistry::get_instance()
 std::string Integer::repr() const
 {
   char s[100];
-  g_snprintf(s, sizeof(s), "%li", _value);
+  g_snprintf(s, sizeof(s), "%li", (long int)_value);
   return s;
 }
 
@@ -110,8 +108,10 @@ Integer* Integer::get(storage_type value)
   static Integer* one= (Integer*)((new Integer(1))->retain());
   static Integer* zero= (Integer*)((new Integer(0))->retain());
 
-  if (value == 1) return one;
-  if (value == 0) return zero;
+  if (value == 1)
+    return one;
+  if (value == 0)
+    return zero;
 
   return new Integer(value);
 }
@@ -331,12 +331,6 @@ void List::remove(size_t index)
   _content.erase(_content.begin()+index);
 }
 
-/**
- * Moves the entry at the old index (oi) to the new index (ni). Due to the way this is implemented
- * (which we cannot change without breaking a lot of code) can the new index behave differently
- * depending on whether it is before or after the old index. In the latter case the new index
- * is off by +1 (as the entry is removed without adjusting the index).
- */
 void List::reorder(size_t oi, size_t ni)
 {
   if (oi == ni)
@@ -359,8 +353,8 @@ size_t List::get_index(const ValueRef &value)
   for (std::vector<ValueRef>::const_iterator iter= _content.begin();
        iter != _content.end(); ++iter, ++i)
   {
-    //if (iter->is_same(value)) return i; // FIXED by mx
-    if ( *iter == value ) return i;
+    if ( *iter == value )
+      return i;
   }
   return npos;
 }
@@ -464,7 +458,7 @@ void List::unmark_global() const
 
 void List::reset_references()
 {
-  const int max_index = count();
+  const int max_index = (int)count();
   grt::ValueRef value;
   for ( int i = 0; i < max_index; ++i )
   {
@@ -601,6 +595,15 @@ ValueRef Dict::operator [](const std::string &key) const
   return iter->second;
 }
 
+Dict::const_iterator Dict::begin() const
+{
+  return _content.begin();
+}
+
+Dict::const_iterator Dict::end() const
+{
+  return _content.end();
+}
 
 std::vector<std::string> Dict::keys() const
 {
