@@ -1158,15 +1158,22 @@ def getTDSProtocolVersionChoices():
 
     return result 
 
-
 @ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection)
 def resetProgressFlags(connection):
     _connections[connection.__id__]['_rev_eng_progress_flags'] = []
     return 0
 
-
 @ModuleInfo.export(grt.INT)
 def cleanup():
     global _connections
     _connections = {}
     return 0
+
+@ModuleInfo.export(grt.STRING, grt.classes.db_mgmt_Connection)
+def getOS(connection):
+    return 'windows'
+
+@ModuleInfo.export(grt.STRING, grt.classes.db_mgmt_Connection)
+def getSourceInstance(connection):
+    return execute_query(connection, "SELECT @@servicename ").fetchone()[0]
+
