@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,17 @@
  * 02110-1301  USA
  */
 
+#include <glib.h>
+#include <boost/signals2.hpp>
+#include <cctype>
 
-#include "stdafx.h"
+#ifdef _WIN32
+#define NOMINMAX
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#endif
+
+#include "base/util_functions.h"
 
 #include "mysql_sql_parser.h"
 #include "grtsqlparser/module_utils.h"
@@ -178,7 +187,7 @@ int Mysql_sql_parser::parse_sql_script(db_CatalogRef &catalog, const std::string
   db_mysql_SchemaRef default_schema;
   int initial_schemata_count= -1;
 
-  initial_schemata_count= _catalog->schemata().count();
+  initial_schemata_count = (int)_catalog->schemata().count();
   if (0 == initial_schemata_count)
     default_schema= set_active_schema("default_schema"); // this will add schema to schemata if necessary
   else
@@ -617,7 +626,7 @@ db_mysql_SchemaRef Mysql_sql_parser::ensure_schema_created(const std::string &sc
     schema= db_mysql_SchemaRef(_grt);
     schema->owner(_catalog);
 
-    std::string time= bec::fmttime(0, DATETIME_FMT);
+    std::string time= base::fmttime(0, DATETIME_FMT);
     schema->createDate(time);
     schema->lastChangeDate(time);
 
@@ -1906,7 +1915,7 @@ Mysql_sql_parser::process_create_routine_statement(const SqlAstNode *tree)
       ListRef<db_mysql_RoutineParam> params= obj->params();
       
       if (_reusing_existing_obj)
-        for (int n= params.count(); n > 0; --n)
+        for (ssize_t n= params.count(); n > 0; --n)
           params.remove_value(params[n-1]);
 
       if (param_list_item)
@@ -2338,9 +2347,11 @@ Mysql_sql_parser::process_create_trigger_statement(const SqlAstNode *tree)
   // event
   SET_STR_SI(obj->event, trigger_tail, sql::_trg_event)
 
+  /*
   // orientation
   if (trigger_tail->find_subseq(sql::_FOR_SYM, sql::_EACH_SYM, sql::_ROW_SYM))
     obj->orientation("ROW");
+    */
 
   // enabled
   obj->enabled(1);
@@ -2638,7 +2649,7 @@ void Mysql_sql_parser::blame_existing_obj(bool critical, const GrtNamedObjectRef
 template <typename T>
 grt::Ref<T> Mysql_sql_parser::create_or_find_named_obj(const grt::ListRef<T>& obj_list, const std::string &obj_name, bool case_sensitive, const GrtNamedObjectRef &container1, const GrtNamedObjectRef &container2)
 {
-  std::string time= bec::fmttime(0, DATETIME_FMT);
+  std::string time= base::fmttime(0, DATETIME_FMT);
 
   grt::Ref<T> obj;
   if (grt::Ref<T>::can_wrap(get_active_object()))
@@ -2671,7 +2682,7 @@ grt::Ref<T> Mysql_sql_parser::create_or_find_named_obj(const grt::ListRef<T>& ob
 template <typename T>
 grt::Ref<T> Mysql_sql_parser::create_or_find_named_routine(const grt::ListRef<T>& obj_list, const std::string &obj_name, bool case_sensitive, const std::string &routine_type, const GrtNamedObjectRef &container1, const GrtNamedObjectRef &container2)
 {
-  std::string time= bec::fmttime(0, DATETIME_FMT);
+  std::string time= base::fmttime(0, DATETIME_FMT);
   
   grt::Ref<T> obj;
   if (grt::Ref<T>::can_wrap(get_active_object()))
