@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@ class MySQLTableEditorBE;
 - (void) moveColumnAtRow: (NSInteger) from
                    toRow: (NSInteger) to;
 
-- (id) initWithListModel: (bec::ListModel*) model
+- (instancetype) initWithListModel: (bec::ListModel*) model
             tableBackEnd: (MySQLTableEditorBE*) tableBackend;
 
 - (void) moveColumnAtRow: (NSInteger) from
