@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -16,11 +16,11 @@
 # 02110-1301  USA
 
 from workbench.db_utils import QueryError, escape_sql_string
+from workbench.utils import Version
 from wb_common import PermissionDeniedError
 
-import os
-from grt import log_error
-_this_file = os.path.basename(__file__)
+import mforms
+from workbench.log import log_error
 
 
 class WBSecurityValidationError(Exception):
@@ -35,11 +35,6 @@ ZOMBIE_TABLE_PRIVS_QUERY = "SELECT t.User, t.Host, t.Db, t.Table_name FROM mysql
 ZOMBIE_COLUMN_PRIVS_QUERY = "SELECT c.User, c.Host, c.Db, c.Table_name, c.Column_name FROM mysql.columns_priv AS c LEFT JOIN mysql.user AS u ON c.User = u.user AND c.Host = u.Host WHERE u.User IS NULL"
 ZOMBIE_PROCS_PRIVS_QUERY = "SELECT p.User, p.Host, p.Db, p.Routine_name, p.Routine_type FROM mysql.procs_priv AS p LEFT JOIN mysql.user AS u ON p.User = u.user AND p.Host = u.Host WHERE u.User IS NULL"
 
-DROP_ZOMBIE_SCHEMA_PRIVS_QUERY = "DELETE FROM mysql.db WHERE User = '%(user)s' AND Host = '%(host)s'"
-DROP_ZOMBIE_TABLE_PRIVS_QUERY = "DELETE FROM mysql.tables_priv WHERE User = '%(user)s' AND Host = '%(host)s'"
-DROP_ZOMBIE_COLUMN_PRIVS_QUERY =  "DELETE FROM mysql.columns_priv WHERE User = '%(user)s' AND Host = '%(host)s'"
-DROP_ZOMBIE_PROCS_PRIVS_QUERY =  "DELETE FROM mysql.procs_priv WHERE User = '%(user)s' AND Host = '%(host)s'"
-
 
 GET_ACCOUNT_QUERY = "SELECT * FROM mysql.user WHERE User='%(user)s' AND Host='%(host)s' ORDER BY User, Host"
 
@@ -58,6 +53,8 @@ GRANT_LIMITS_QUERY = "GRANT USAGE ON *.* TO '%(user)s'@'%(host)s' WITH %(limit)s
 RENAME_USER_QUERY = "RENAME USER '%(old_user)s'@'%(old_host)s' TO '%(user)s'@'%(host)s'"
 CHANGE_PASSWORD_QUERY = "SET PASSWORD FOR '%(user)s'@'%(host)s' = PASSWORD('%(password)s')"
 BLANK_PASSWORD_QUERY = "SET PASSWORD FOR '%(user)s'@'%(host)s' = ''"
+CHANGE_PASSWORD_QUERY_576 = "ALTER USER '%(user)s'@'%(host)s' IDENTIFIED BY '%(password)s'"
+BLANK_PASSWORD_QUERY_576 = "ALTER USER '%(user)s'@'%(host)s' IDENTIFIED BY ''"
 
 REVOKE_SCHEMA_PRIVILEGES_QUERY = "REVOKE %(revoked_privs)s ON `%(db)s`.* FROM '%(user)s'@'%(host)s'"
 GRANT_SCHEMA_PRIVILEGES_QUERY = "GRANT %(granted_privs)s ON `%(db)s`.* TO '%(user)s'@'%(host)s'"
@@ -334,7 +331,7 @@ class AdminSecurity(object):
             try:
                 result = self.ctrl_be.exec_query(query)
             except Exception, e:
-                log_error(_this_file, "Could not get list of invalid privs: %s\nQuery: %s\n" % (e, query))
+                log_error("Could not get list of invalid privs: %s\nQuery: %s\n" % (e, query))
                 return []
             privs = []
             while result and result.nextRow():
@@ -381,21 +378,25 @@ class AdminSecurity(object):
         return self._accounts
 
 
+    def do_delete_account(self, username, host):
+        query = REMOVE_USER % {"user":escape_sql_string(username), "host":escape_sql_string(host)}
+        try:
+            self.ctrl_be.exec_sql("use mysql")
+            self.ctrl_be.exec_sql(query)
+        except QueryError, e:
+            log_error('Error removing account %s@%s:\n%s' % (username, host, str(e)))
+            if e.error == 1227: # MySQL error code 1227 (ER_SPECIFIC_ACCESS_DENIED_ERROR)
+                raise Exception('Error removing the account  %s@%s:' % (username, host),
+                                'You must have the global CREATE USER privilege or the DELETE privilege for the mysql '
+                                'database')
+            raise e
+
     def delete_account(self, account):
         if account.is_commited:
-            query = REMOVE_USER % {"user":escape_sql_string(account.username), "host":escape_sql_string(account.host)}
-            try:
-                self.ctrl_be.exec_sql("use mysql")
-                self.ctrl_be.exec_sql(query)
-            except QueryError, e:
-                if e.error == 1227:
-                    raise Exception('Error removing the account  %s@%s:' % (account.username, account.host),
-                                    'You must have the global CREATE USER privilege or the DELETE privilege for the mysql '
-                                    'database')
+            self.do_delete_account(account.username, account.host)
         del self._account_info_cache[account.username+"@"+account.host]
         if (account.username, account.host) in self._accounts:
-            self._accounts.remove((account.username, account.host))
-
+          self._accounts.remove((account.username, account.host))
 
     def revert_account(self, account, backup):
         try:
@@ -443,41 +444,23 @@ class AdminSecurity(object):
         return self._zombie_privs.get((user,host), None)
 
 
-    def wipe_zombie(self, user, host):
-        try:
-            self.ctrl_be.exec_query(DROP_ZOMBIE_SCHEMA_PRIVS_QUERY % {'user':escape_sql_string(user), 'host':escape_sql_string(host)})
-        except Exception, e:
-            log_error(_this_file, "Could not delete schema privileges for %s@%s: %s\n" % (user, host, e))
-        try:
-            self.ctrl_be.exec_query(DROP_ZOMBIE_TABLE_PRIVS_QUERY % {'user':escape_sql_string(user), 'host':escape_sql_string(host)})
-        except Exception, e:
-            log_error(_this_file, "Could not delete table privileges for %s@%s: %s\n" % (user, host, e))
-        try:
-            self.ctrl_be.exec_query(DROP_ZOMBIE_COLUMN_PRIVS_QUERY % {'user':escape_sql_string(user), 'host':escape_sql_string(host)})
-        except Exception, e:
-            log_error(_this_file, "Could not delete column privileges for %s@%s: %s\n" % (user, host, e))
-        try:
-            self.ctrl_be.exec_query(DROP_ZOMBIE_PROCS_PRIVS_QUERY % {'user':escape_sql_string(user), 'host':escape_sql_string(host)})
-        except Exception, e:
-            log_error(_this_file, "Could not delete procs privileges for %s@%s: %s\n" % (user, host, e))
-
-
-
-
-
     def async_get_account(self, callback, name, host):
-        key = name+"@"+host
+        try:
+          key = name+"@"+host
 
-        if self._account_info_cache.has_key(key):
-            callback(self._account_info_cache[key])
-            return
+          if self._account_info_cache.has_key(key):
+              callback(self._account_info_cache[key])
+              return
 
-        account = AdminAccount(self)
-        account.load(name, host)
+          account = AdminAccount(self)
+          account.load(name, host)
 
-        self._account_info_cache[key] = account
+          self._account_info_cache[key] = account
 
-        callback(account)
+          callback(account)
+        except:
+          mforms.Utilities.driver_shutdown()
+          raise
 
 
 
@@ -566,8 +549,6 @@ class AdminUserDbPrivs(object):
         # workaround for server bug
         self._owner.ctrl_be.exec_sql("use mysql")
 
-        handled_entries = set()
-
         for deleted, entry in [(True, e) for e in self._deleted_entries] + [(False, e) for e in self.entries]:
 
             fields = { "user":escape_sql_string(self._owner.username),
@@ -581,7 +562,7 @@ class AdminUserDbPrivs(object):
                 (priv_name, description) = PrivilegeInfo.get(priv, (None,None))
                 if not priv_name:
                     continue
-                if not deleted and priv in entry.privileges:
+                if (not deleted and priv in entry.privileges) or not entry.privileges:
                     granted_privs.append(priv_name)
                 else:
                     revoked_privs.append(priv_name)
@@ -590,7 +571,7 @@ class AdminUserDbPrivs(object):
                 try:
                     self._owner.ctrl_be.exec_sql(GRANT_SCHEMA_PRIVILEGES_QUERY % fields)
                 except QueryError, e:
-                    if e.error == 1045:
+                    if e.error in [1045, 1044]:
                         raise Exception('Error assigning privileges for %(user)s@%(host)s in schema %(db)s' % fields,
                                         'You must have the GRANT OPTION privilege, and you must have the privileges that you are granting')
                     raise e
@@ -600,7 +581,7 @@ class AdminUserDbPrivs(object):
                 try:
                     self._owner.ctrl_be.exec_sql(REVOKE_SCHEMA_PRIVILEGES_QUERY % fields)
                 except QueryError, e:
-                    if e.error == 1044:
+                    if e.error in [1045, 1044]:
                         raise Exception('Error revoking privileges for %(user)s@%(host)s in schema %(db)s' % fields,
                                         'You must have the GRANT OPTION privilege, and you must have the privileges that you are revoking')
                     raise e
@@ -627,6 +608,8 @@ class AdminAccount(object):
     max_user_connections = 0
     auth_plugin = None
     auth_string = None
+    old_authentication = False
+    blank_password = False
 
     _global_privs = set()
     _custom_checked_privs = set()
@@ -845,15 +828,17 @@ class AdminAccount(object):
                 queries.append(REVOKE_ALL % fields)
 
         if self.password != self._orig_password and not password_already_set:
+            change_pw = CHANGE_PASSWORD_QUERY if self._owner.ctrl_be.target_version and self._owner.ctrl_be.target_version < Version(5,7,6) else CHANGE_PASSWORD_QUERY_576
+            blank_pw = BLANK_PASSWORD_QUERY if self._owner.ctrl_be.target_version and self._owner.ctrl_be.target_version < Version(5,7,6) else BLANK_PASSWORD_QUERY
             # special hack required by server to handle sha256 password accounts
             if self.auth_plugin == "sha256_password":
                 queries.append("SET old_passwords = 2")
             else:
                 queries.append("SET old_passwords = 0")
             if fields["password"]:
-                queries.append(CHANGE_PASSWORD_QUERY % fields)
+                queries.append(change_pw % fields)
             else:
-                queries.append(BLANK_PASSWORD_QUERY % fields)
+                queries.append(blank_pw % fields)
 
         action = "changing" if self.is_commited else "creating"
         for query in queries:
@@ -946,6 +931,10 @@ class AdminAccount(object):
         if self._owner.has_password_expired:
             self.password_expired = result.stringByName("password_expired") == 'Y'
 
+        password_column = 'password' if self._owner.ctrl_be.target_version and self._owner.ctrl_be.target_version < Version(5,7,6) else 'authentication_string'
+        self.old_authentication = len(result.stringByName(password_column)) == 16
+        self.blank_password = len(result.stringByName(password_column)) == 0
+
         self._global_privs = set()
         for priv in self._owner.global_privilege_names:
             if result.stringByName(priv) == 'Y':
@@ -1000,3 +989,38 @@ class AdminAccount(object):
         self._orig_global_privs = self._global_privs.copy()
 
         self.schema_privs.load()
+
+    def upgrade_password_format(self):
+        queries = []
+        if self.password != self.confirm_password:
+            raise WBSecurityValidationError("The new password and its confirmation don't match. Please re-enter them.")
+
+        queries.append("use mysql")
+
+        if not self.host:
+            raise WBSecurityValidationError("Host name must not be blank")
+
+        fields = {
+            "user" : escape_sql_string(self.username) or "NULL",
+            "host" : escape_sql_string(self.host) or "",
+            "password" : escape_sql_string(self.password or ""),
+        }
+
+        queries.append("SET old_passwords = 0")        
+        if self._owner.ctrl_be.target_version and self._owner.ctrl_be.target_version.is_supported_mysql_version_at_least(5, 5, 7):
+            queries.append("UPDATE mysql.user SET plugin = 'mysql_native_password' WHERE user = '%(user)s' AND host = '%(host)s'" % fields)
+        queries.append("FLUSH PRIVILEGES")
+        queries.append("SET PASSWORD FOR '%(user)s'@'%(host)s' = PASSWORD('%(password)s')" % fields)
+        queries.append("FLUSH PRIVILEGES")
+        
+        action = "changing"
+        for query in queries:
+            try:
+                self._owner.ctrl_be.exec_sql(query)
+            except QueryError, e:
+                if e.error == 1142:
+                    raise Exception("Error %s account %s@%s: Insufficient rights to perform operation"%(action, self.username, self.host))
+                else:
+                    raise Exception("Error %s account %s@%s: %s"%(action, self.username, self.host, e.errortext or e))
+            except Exception, e:
+                raise Exception("Error %s account %s@%s: %s"%(action, self.username, self.host, e))
